#include <stdint.h>

#include <iostream>

#include <clang/AST/ASTConsumer.h>
#include <clang/AST/ASTContext.h>
#include <clang/AST/DeclGroup.h>
#include <clang/AST/TypeLoc.h>
#include <clang/Basic/FileManager.h>
#include <clang/Basic/TargetInfo.h>
#include <clang/Basic/TargetOptions.h>
#include <clang/Frontend/DiagnosticOptions.h>
#include <clang/Frontend/FrontendOptions.h>
#include <clang/Frontend/HeaderSearchOptions.h>
#include <clang/Frontend/PreprocessorOptions.h>
#include <clang/Frontend/TextDiagnosticPrinter.h>
#include <clang/Frontend/Utils.h>
#include <clang/Lex/HeaderSearch.h>
#include <clang/Lex/Preprocessor.h>
#include <clang/Sema/ParseAST.h>


using namespace clang;
using namespace std;

SourceManager* source_manager;

class MyConsumer : public ASTConsumer
{
public:
	virtual void HandleTopLevelDecl(DeclGroupRef D);
};

void MyConsumer::HandleTopLevelDecl(DeclGroupRef group)
{
        for(DeclGroupRef::iterator i=group.begin(); i!=group.end(); i++)
        {
                Decl* decl=*i;
                switch(decl->getKind())
                {
                        case Decl::Function:
                        {
                                FunctionDecl* function=static_cast<FunctionDecl*>(decl);
				VarDecl* parm=function->getParamDecl(0);
				SourceLocation pos;
				pos=parm->getSourceRange().getBegin();  // Nope, didn't work
				cout << "Parameter declaration starts at column " << source_manager->getSpellingColumnNumber(pos) << endl;
				pos=parm->getTypeSpecStartLoc();  // Nope, didn't work wither
				cout << "Parameter declaration starts at column " << source_manager->getSpellingColumnNumber(pos) << endl;
				pos=parm->getTypeSourceInfo()->getTypeLoc().getFullSourceRange().getBegin();
				cout << "Parameter declaration starts at column " << source_manager->getSpellingColumnNumber(pos) << endl;
			}
		}
	}
}

int main(int argc, char* argv[])
{
	llvm::raw_ostream* ostream=new llvm::raw_stderr_ostream();
	DiagnosticOptions diagnostic_options=DiagnosticOptions();
	DiagnosticClient* diag_client=new TextDiagnosticPrinter(*ostream, diagnostic_options, false);
	Diagnostic diagnostic=Diagnostic(diag_client);

	LangOptions options=LangOptions();

	TargetOptions target_options=TargetOptions();
	target_options.Triple=LLVM_HOSTTRIPLE;
	TargetInfo* target_info=TargetInfo::CreateTargetInfo(diagnostic, target_options);

	FileManager file_manager;
	const FileEntry* file=file_manager.getFile("const_pos.c");
	if(file==0)
	{
		cerr << "Failed to find input file" << endl;
		return false;
	}

	source_manager=new SourceManager(diagnostic);
        source_manager->createMainFileID(file, SourceLocation());
	HeaderSearch header_search(file_manager);
	Preprocessor preprocessor(
		diagnostic, options, *target_info, *source_manager,
		header_search
	);

	MyConsumer consumer=MyConsumer();
	diag_client->BeginSourceFile(options, &preprocessor);
	PreprocessorOptions preprocessor_options=PreprocessorOptions();
	HeaderSearchOptions header_search_options=HeaderSearchOptions();
	FrontendOptions frontend_options=FrontendOptions();
	InitializePreprocessor(preprocessor, preprocessor_options, header_search_options, frontend_options);


	IdentifierTable identifier_table(options);
	SelectorTable selector_table;
	Builtin::Context builtins=Builtin::Context(*target_info);
	ASTContext context(options, *source_manager, *target_info, identifier_table, selector_table, builtins);
	ParseAST(preprocessor, &consumer, context, false, true);

	delete source_manager;
	delete target_info;
	delete diag_client;
	delete ostream;
	return 0;
}
